@@ -21,7 +21,13 @@ from twine.cli import dispatch
 
 
 def main():
-    return dispatch(sys.argv[1:])
+    try:
+        return dispatch(sys.argv[1:])
+    except Exception as exc:
+        return '{0}: {1}'.format(
+            exc.__class__.__name__,
+            exc.args[0],
+        )
 
 
 if __name__ == "__main__":
