@@ -21,7 +21,7 @@ import sys
 
 import twine.exceptions as exc
 from twine.package import PackageFile
-from twine.repository import Repository
+from twine.repository import Repository, LEGACY_PYPI
 from twine import utils
 
 
@@ -72,7 +72,8 @@ def skip_upload(response, skip_existing, package):
 
 
 def upload(dists, repository, sign, identity, username, password, comment,
-           sign_with, config_file, skip_existing, cert, client_cert):
+           sign_with, config_file, skip_existing, cert, client_cert,
+           repository_url):
     # Check that a nonsensical option wasn't given
     if not sign and identity:
         raise ValueError("sign must be given along with identity")
@@ -85,7 +86,11 @@ def upload(dists, repository, sign, identity, username, password, comment,
     )
     uploads = [i for i in dists if not i.endswith(".asc")]
 
-    config = utils.get_repository_from_config(config_file, repository)
+    config = utils.get_repository_from_config(
+        config_file,
+        repository,
+        repository_url,
+    )
 
     config["repository"] = utils.normalize_repository_url(
         config["repository"]
@@ -93,8 +98,16 @@ def upload(dists, repository, sign, identity, username, password, comment,
 
     print("Uploading distributions to {0}".format(config["repository"]))
 
+    if config["repository"].startswith(LEGACY_PYPI):
+        print(
+            "Note: you are uploading to the old upload URL. It's recommended "
+            "to use the new URL \"{0}\" or to leave the URL unspecified and "
+            "allow twine to choose.".format(utils.DEFAULT_REPOSITORY))
+
     username = utils.get_username(username, config)
-    password = utils.get_password(password, config)
+    password = utils.get_password(
+        config["repository"], username, password, config,
+    )
     ca_cert = utils.get_cacert(cert, config)
     client_cert = utils.get_clientcert(client_cert, config)
 
@@ -138,7 +151,7 @@ def upload(dists, repository, sign, identity, username, password, comment,
             print(skip_message)
             continue
 
-        resp.raise_for_status()
+        utils.check_status_code(resp)
 
     # Bug 28. Try to silence a ResourceWarning by clearing the connection
     # pool.
@@ -149,8 +162,24 @@ def main(args):
     parser = argparse.ArgumentParser(prog="twine upload")
     parser.add_argument(
         "-r", "--repository",
+        action=utils.EnvironmentDefault,
+        env="TWINE_REPOSITORY",
         default="pypi",
-        help="The repository to upload the files to (default: %(default)s)",
+        help="The repository to register the package to. Can be a section in "
+             "the config file or a full URL to the repository (default: "
+             "%(default)s). (Can also be set via %(env)s environment "
+             "variable)",
+    )
+    parser.add_argument(
+        "--repository-url",
+        action=utils.EnvironmentDefault,
+        env="TWINE_REPOSITORY_URL",
+        default=None,
+        required=False,
+        help="The repository URL to upload the package to. This can be "
+             "specified with --repository because it will be used if there is "
+             "no configuration for the value passed to --repository. "
+             "(Can also be set via %(env)s environment variable.)"
     )
     parser.add_argument(
         "-s", "--sign",
@@ -169,11 +198,19 @@ def main(args):
     )
     parser.add_argument(
         "-u", "--username",
-        help="The username to authenticate to the repository as",
+        action=utils.EnvironmentDefault,
+        env="TWINE_USERNAME",
+        required=False, help="The username to authenticate to the repository "
+                             "as (can also be set via %(env)s environment "
+                             "variable)",
     )
     parser.add_argument(
         "-p", "--password",
-        help="The password to authenticate to the repository with",
+        action=utils.EnvironmentDefault,
+        env="TWINE_PASSWORD",
+        required=False, help="The password to authenticate to the repository "
+                             "with (can also be set via %(env)s environment "
+                             "variable)",
     )
     parser.add_argument(
         "-c", "--comment",
@@ -194,14 +231,19 @@ def main(args):
     )
     parser.add_argument(
         "--cert",
+        action=utils.EnvironmentDefault,
+        env="TWINE_CERT",
+        default=None,
+        required=False,
         metavar="path",
-        help="Path to alternate CA bundle",
+        help="Path to alternate CA bundle (can also be set via %(env)s "
+             "environment variable)",
     )
     parser.add_argument(
         "--client-cert",
         metavar="path",
         help="Path to SSL client certificate, a single file containing the "
-             "private key and the certificate in PEM forma",
+             "private key and the certificate in PEM format",
     )
     parser.add_argument(
         "dists",
@@ -215,10 +257,7 @@ def main(args):
     args = parser.parse_args(args)
 
     # Call the upload function with the arguments from the command line
-    try:
-        upload(**vars(args))
-    except Exception as exc:
-        sys.exit("{exc.__class__.__name__}: {exc}".format(exc=exc))
+    upload(**vars(args))
 
 
 if __name__ == "__main__":
